///////////////////////////////////////////////////////////
// DIAGBMP.CPP
///////////////////////////////////////////////////////////

#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dialog.h>
#include <owl\button.h>
#include <owl\gdiobject.h>
#include <stdlib.h>
#pragma hdrstop
#include "diagbmp.rc"

#define BITMAP_W 129
#define BITMAP_H 97


//////////////////////////////////
// The dialog class.
//////////////////////////////////
class TDlg : public TDialog
{
public:
	TDlg(TWindow *parent, TResId resId);
	~TDlg();
protected:
	BOOL EvEraseBkgnd(HDC hDC);
private:
	void CmShowBitmap1();
	void CmShowBitmap2();
	void CmHideBitmap1();
	void CmHideBitmap2();
	void CmEnableShowBitmap(TCommandEnabler &ce);
	void CmEnableHideBitmap(TCommandEnabler &ce);
	void Paint(TDC &dc, BOOL erase, TRect& rect);
	void EvPaint();

	TBitmap *bitmap;
	BOOL bitmapShown;
DECLARE_RESPONSE_TABLE(TDlg);
};

DEFINE_RESPONSE_TABLE1(TDlg, TDialog)
	EV_WM_ERASEBKGND,
	EV_WM_PAINT,
	EV_COMMAND(CM_SHOWBITMAP1, CmShowBitmap1),
	EV_COMMAND(CM_SHOWBITMAP2, CmShowBitmap2),
	EV_COMMAND(CM_HIDEBITMAP1, CmHideBitmap1),
	EV_COMMAND(CM_HIDEBITMAP2, CmHideBitmap2),
	EV_COMMAND_ENABLE(CM_SHOWBITMAP1, CmEnableShowBitmap),
	EV_COMMAND_ENABLE(CM_SHOWBITMAP2, CmEnableShowBitmap),
	EV_COMMAND_ENABLE(CM_HIDEBITMAP1, CmEnableHideBitmap),
	EV_COMMAND_ENABLE(CM_HIDEBITMAP2, CmEnableHideBitmap),
END_RESPONSE_TABLE;


TDlg::TDlg(TWindow *parent, TResId resId)
	: TDialog(parent, resId),
		bitmapShown(FALSE)
{
	bitmap = new TBitmap(GetApplication()->GetInstance(), BITMAP_1);
}


TDlg::~TDlg()
{
	delete bitmap;
}


BOOL TDlg::EvEraseBkgnd(HDC hDC)
{
	TDC dc(hDC);
	TBrush black(TColor::Black);
	dc.SelectObject(black);
	dc.PatBlt(0, 0, TWindow::Attr.W, TWindow::Attr.H);

	return TRUE;
}


void TDlg::EvPaint()
{
	TWindow::EvPaint();
}


void TDlg::Paint(TDC &dc, BOOL erase, TRect& rect)
{
	TWindow::Paint(dc, erase, rect);
	if (bitmapShown)
  	CmShowBitmap1();
}


void TDlg::CmShowBitmap1()
{
	TClientDC dc(HWindow);
	TMemoryDC memdc(dc);
	memdc.SelectObject(*bitmap);

	dc.BitBlt(
		80, 40,     					//destination x, y
		BITMAP_W, BITMAP_H,		//bitmap width & height
		memdc,      					//source device context
		0, 0,       					//source X,Y
		SRCCOPY);   					//raster display operation

	memdc.RestoreBitmap();   // Not strictly necessary
	bitmapShown = TRUE;
}


void TDlg::CmShowBitmap2()
{
	TClientDC dc(HWindow);
	TMemoryDC memdc(dc);
	memdc.SelectObject(*bitmap);

	int bitmapSize = BITMAP_W * BITMAP_H;		//get the bitmap size
	int *bitArray = new int[bitmapSize];		//make an array
	for (int i = 0; i < bitmapSize; i++)		//and fill it with 0, 1, 2, ...
		bitArray[i] = i;

	randomize();														//just to make it fun
	int index, x, y, randPick;
	for (i = 0; i < bitmapSize; i++) {
		randPick = random(bitmapSize - i);		//pick one from the array
		index = bitArray[randPick];						//and save the value
		bitArray[randPick] = bitArray[bitmapSize - 1 - i];	//shrink array by 1

		x = index % BITMAP_W;
		y = index / BITMAP_W;
		dc.SetPixel(x + 80, y + 40, memdc.GetPixel(x, y));
	}

	delete[] bitArray;
	bitmapShown = TRUE;
}


void TDlg::CmHideBitmap1()
{
	Invalidate();
	bitmapShown = FALSE;
}


void TDlg::CmHideBitmap2()
{
	TClientDC dc(HWindow);

	int bitmapSize = BITMAP_W * BITMAP_H;		//get the bitmap size
	int *bitArray = new int[bitmapSize];		//make an array
	for (int i = 0; i < bitmapSize; i++)		//and fill it with 0, 1, 2, ...
		bitArray[i] = i;

	randomize();														//just to make it fun
	int index, x, y, randPick;
	for (i = 0; i < bitmapSize; i++) {
		randPick = random(bitmapSize - i);		//pick one from the array
		index = bitArray[randPick];						//and save the value
		bitArray[randPick] = bitArray[bitmapSize - 1 - i];	//shrink array by 1

		x = index % BITMAP_W;
		y = index / BITMAP_W;
		dc.SetPixel(x + 80, y + 40, TColor::Black);
	}

	delete[] bitArray;
	bitmapShown = FALSE;
}



void TDlg::CmEnableShowBitmap(TCommandEnabler &ce)
{
	ce.Enable(!bitmapShown);
}

void TDlg::CmEnableHideBitmap(TCommandEnabler &ce)
{
	ce.Enable(bitmapShown);
}

//////////////////////////////////
// The application class.
//////////////////////////////////
class TApp : public TApplication
{
public:
	TApp(): TApplication() {}
	void InitMainWindow();
};

///////////////////////////////////////////////////////////
// TApp::InitMainWindow()
///////////////////////////////////////////////////////////
void TApp::InitMainWindow()
{
	// Construct the dialog box.
	TDialog *dialog = new TDlg(0, TESTDLG);

	// Construct the main window with the
	// dialog box as the client window.
	TFrameWindow *wndw =
		new TFrameWindow(0, "Bitmap in Dialog",
		dialog, TRUE);

	// Assign a menu to the main window.
	wndw->AssignMenu(MENU_1);
	wndw->Attr.Style &= ~(WS_MAXIMIZEBOX | WS_THICKFRAME);

	SetMainWindow(wndw);
}

///////////////////////////////////////////////////////////
// OwlMain()
///////////////////////////////////////////////////////////
int OwlMain(int, char*[])
{
   return TApp().Run();
}


